package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.productservice.repository.SupportRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class SupportService {

    @Autowired
    private SupportRepository supportRepository;

    public boolean exists(String code) {
        Long count = supportRepository.countByCode(code);
        return count > 0;
    }

    public APIResult<List<Support>> findByIds(List<String> supportIdList) {
        List<Support> supportList = supportRepository.findByIdIn(supportIdList, BaseStatusEnum.NORMAL);
        if (supportList == null || supportList.size() == 0) {
            return APIResult.ok(supportList);
        }

        Map<String, Support> supportMap = ListUtil.convert(supportList, "id", Support.class);
        List<Support> sortSupportList = new ArrayList<>();
        supportIdList.forEach( id -> {
            Support support = supportMap.get(id);
            if (support != null) {
                sortSupportList.add(support);
            }
        });

        return APIResult.ok(supportList);
    }

    public APIResult<Page<Support>> search(Support support, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = supportRepository.count(support);
        Page<Support> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Support> supportList = supportRepository.find(support, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(supportList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Support> create(SupportDTO supportDTO) {
        String code = supportDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Support support = convert(supportDTO);
        Support result = supportRepository.save(support);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Support> update(SupportDTO supportDTO) {

        Support support = convert(supportDTO);
        Support result = supportRepository.update(support);
        
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = supportRepository.updateStatusById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }

        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<Support> findById(String id) {
        Support support = supportRepository.find(id);
        if (support == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }
        return APIResult.ok(support);
    }

}
