package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.TagDTO;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.product.entity.Tag;
import cn.rankin.productservice.repository.TagRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class TagService {

    @Autowired
    private TagRepository tagRepository;

    public APIResult<Tag> findById(String id) {
        Tag tag = tagRepository.find(id);
        return APIResult.ok(tag);
    }

    public APIResult<List<Tag>> findByIds(List<String> tagIdList) {
        List<Tag> tagList = tagRepository.findByIdIn(tagIdList, BaseStatusEnum.NORMAL);
        return APIResult.ok(tagList);
    }

    public APIResult<Page<Tag>> search(Tag tag, Integer pageNo, Integer pageSize) {
        Long count = tagRepository.count(tag);
        Page<Tag> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Tag> tagList = tagRepository.search(tag, Long.valueOf(page.getStart()).intValue(), pageSize);
//        List<Tag> tagList = tagRepository.find(tag, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(tagList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Tag> create(TagDTO tagDTO) {

        Tag tag = convert(tagDTO);
        Tag result = tagRepository.save(tag);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Tag> update(TagDTO tagDTO) {

        Tag tag = convert(tagDTO);
        Tag result = tagRepository.update(tag);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = tagRepository.updateStatusById(id, BaseStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    @Transactional
    public APIResult<Boolean> deleteByGroupId(String groupId) {
        tagRepository.deleteByGroupId(groupId, BaseStatusEnum.DEL);
        return APIResult.ok(true);
    }

    public APIResult<List<Tag>> findNormalByGroupId(String groupId) {
        List<Tag> tagList = tagRepository.findNormalByGroupId(groupId, BaseStatusEnum.NORMAL);
        return APIResult.ok(tagList);
    }

    @Transactional
    public APIResult<List<Tag>> updateGroupRelation(String groupId, List<TagDTO> tagDTOList) {
        if (tagDTOList == null) {
            return APIResult.ok();
        }

        deleteByGroupId(groupId);

        if (tagDTOList.size() == 0) {
            return APIResult.ok(new ArrayList<>());
        }

        List<String> tagIdList = new ArrayList<>();
        List<Tag> tagList = new ArrayList<>();
        for (int i = 0; i < tagDTOList.size(); i++) {
            TagDTO dto = tagDTOList.get(i);
            if (dto == null) {
                continue;
            }

            String tagId = dto.getId();
            if (!StringUtils.isEmpty(tagId)) {
                tagIdList.add(tagId);
            }

            Tag tag = convert(dto);
            tag.setGroupId(groupId);
            tag.setSort(i);
            tag.setStatus(BaseStatusEnum.NORMAL);

            tagList.add(tag);
        }

        List<Tag> result = tagRepository.save(tagList);

        return APIResult.ok(result);

    }

}
